@@ -306,11 +306,12 @@ def remediate_etc(imgbase):
     def check_layers(m, n):
         candidates = find_problems(m.path("/"), n.path("/"))
         for c in sorted(candidates):
-            copy_from = n.path("/usr/share/factory") + c
-            copy_to = n.path("/") + c
+            if "targeted/active/modules" not in c:
+                copy_from = n.path("/usr/share/factory") + c
+                copy_to = n.path("/") + c
 
-            log.debug("Copying %s to %s" % (copy_from, copy_to))
-            shutil.copy2(copy_from, copy_to)
+                log.debug("Copying %s to %s" % (copy_from, copy_to))
+                shutil.copy2(copy_from, copy_to)
 
     tree = imgbase.naming.tree()
 
@@ -323,10 +324,12 @@ def remediate_etc(imgbase):
         with mounted(imgbase._lvm_from_layer(layers[idx]).path) as m, \
                 mounted(imgbase._lvm_from_layer(layers[idx+1]).path) as n:
                     # Resync the files we changed on the last pass
-                    r = Rsync(checksum_only=True, update_only=True)
+                    r = Rsync(checksum_only=True, update_only=True,
+                              exclude=["*targeted/active/modules*"])
                     r.sync(m.path("/etc"), n.path("/etc"))
 
                     check_layers(m, n)
+                    fix_systemd_services(m, n)
 
 
 def migrate_etc(imgbase, new_lv, previous_lv):
@@ -413,8 +416,10 @@ def migrate_etc(imgbase, new_lv, previous_lv):
             # not actually the same filesystem
             if old_fs.source != new_fs.source:
                 for c in changed:
-                    copy_files(new_fs.path("/") + c, [old_fs.path("/") + c],
-                               "-a", "-r")
+                    if "targeted/active/modules" not in c:
+                        copy_files(new_fs.path("/") + c,
+                                   [old_fs.path("/") + c],
+                                   "-a", "-r")
 
             File(new_fs.path("/etc/group")).write(group_content)
             File(new_fs.path("/etc/passwd")).write(passwd_content)
@@ -433,10 +438,13 @@ def migrate_etc(imgbase, new_lv, previous_lv):
         threads.append(ThreadRunner(run_rpm_perms, new_lv))
         threads.append(ThreadRunner(fix_systemd_services, old_fs, new_fs))
         threads.append(ThreadRunner(run_rpm_selinux_post, new_lv))
-        threads.append(ThreadRunner(relabel_selinux, new_fs))
 
         thread_group_handler(threads)
 
+        # This needs to be called after calling the selinux post install
+        # scripts in case someone added a file context and needs a relabeling
+        relabel_selinux(new_fs)
+
         Motd(new_etc + "/motd").clear_motd()
 
 
@@ -469,29 +477,64 @@ def fix_systemd_services(old_fs, new_fs):
 
     for d in diffs:
         log.debug("Removing %s" % d)
+        filename = os.path.basename(d)
         try:
             if os.path.exists(new_fs.path("/") + d):
                 if os.path.isdir(new_fs.path("/") + d):
                     remove_file(new_fs.path("/") + d, dir=True)
                 elif os.path.isfile(new_fs.path("/") + d):
                     remove_file(new_fs.path("/") + d)
+
+            # EL updates can move some of these around. Firewalld goes from
+            # basic.target.wants to multiuser.target.wants in 7.4. Check the
+            # entire tree
+            for root, dirs, files in os.walk(new_fs.path("/etc/systemd")):
+                for d in dirs:
+                    if d == filename:
+                        log.debug("Found a disabled systemd service "
+                                  "elsewhere. Removing it: %s" % filename)
+                        remove_file(os.path.join(root, d), dir=True)
+                for f in files:
+                    if f == filename:
+                        log.debug("Found a disabled systemd service "
+                                  "elsewhere. Removing it: %s" % filename)
+                        remove_file(os.path.join(root, f))
         except:
             log.exception("Could not remove %s. Is it a read-only layer?")
 
 
 def relabel_selinux(new_fs):
-    fc = "/etc/selinux/targeted/contexts/files/file_contexts"
+    ctx_files = ["/etc/selinux/targeted/contexts/files/file_contexts",
+                 "/etc/selinux/targeted/contexts/files/file_contexts.homedirs",
+                 "/etc/selinux/targeted/contexts/files/file_contexts.local"]
 
-    if not os.path.exists(new_fs.path("/") + fc):
-        log.debug("{} not found in new fs, not relabeling".format(fc))
-        return
+    dirs = ["/etc",
+            "/usr/bin",
+            "/usr/bin",
+            "/usr/sbin",
+            "/usr/share",
+            "/var"]
 
-    dirs = ["/etc", "/var", "/usr/libexec", "/usr/bin", "/usr/sbin"]
+    exclude_dirs = ["/usr/share/factory"]
+
+    # Reduce the list to something subprocess can use directly
+    excludes = sum([["-e", d] for d in exclude_dirs], [])
+
+    new_root = new_fs.path("/")
+
+    log.debug("Relabeling selinux")
 
     with SELinuxDomain("setfiles_t") as dom:
-        for d in dirs:
-            dom.runcon(["setfiles", "-v", "-r", new_fs.path("/"),
-                        fc, new_fs.path("/") + dirs])
+        with utils.bindmounted("/var", target=new_fs.path("/") + "/var",
+                               rbind=True):
+            for fc in ctx_files:
+                if os.path.exists(new_root + "/" + fc):
+                    dom.runcon(["setfiles", "-v", "-r", new_fs.path("/"),
+                                fc] +
+                               excludes +
+                               dirs)
+                else:
+                    log.debug("{} not found in new fs, skipping".format(fc))
 
 
 def run_rpm_selinux_post(new_lv):
@@ -512,11 +555,11 @@ def run_rpm_selinux_post(new_lv):
                                 **kwargs).communicate()
         return proc[0]
 
-    def filter_selinux_commands(rpms):
+    def filter_selinux_commands(rpms, scr_arg):
         for pkg, v in rpms.items():
             if any([c for c in critical_commands if c in v]):
                 log.debug("Found a critical command in %s", pkg)
-                run_commands.append("bash -c '{}'".format(v))
+                run_commands.append("bash -c '{}' -- {}".format(v, scr_arg))
 
     with mounted(new_lv.path) as new_fs:
         log.debug("Checking whether any %post scripts from the new image must "
@@ -527,8 +570,8 @@ def run_rpm_selinux_post(new_lv):
         postin = rpmdb.get_script_type('POSTIN')
         posttrans = rpmdb.get_script_type('POSTTRANS')
 
-        filter_selinux_commands(postin)
-        filter_selinux_commands(posttrans)
+        filter_selinux_commands(postin, 1)
+        filter_selinux_commands(posttrans, 0)
 
         with utils.bindmounted("/proc", new_fs.target + "/proc"):
             with utils.bindmounted("/dev", new_fs.target + "/dev"):
@@ -551,7 +594,7 @@ def relocate_var_lib_yum(new_lv):
 
 def run_rpm_perms(new_lv):
     with mounted(new_lv.path) as new_fs:
-        with utils.bindmounted("/var", new_fs.path("/var")):
+        with utils.bindmounted("/var", new_fs.path("/var"), rbind=True):
             hack_rpm_permissions(new_fs)
 
 
@@ -829,7 +872,8 @@ def adjust_mounts_and_boot(imgbase, new_lv, previous_lv):
         loader.set_default(new_lv.lv_name)
 
     with mounted(new_lv.path) as newroot:
-        with utils.bindmounted("/var", target=newroot.target + "/var"):
+        with utils.bindmounted("/var", target=newroot.target + "/var",
+                               rbind=True):
             update_fstab(newroot.target)
             update_grub_default(newroot.target)
             copy_kernel(newroot.target)
